Package: statgenGWAS
Type: Package
Title: Genome Wide Association Studies
Version: 1.0.13
Date: 2026-02-19
Authors@R: c(person(given = "Bart-Jan", 
                    family = "van Rossum", 
                    email = "bart-jan.vanrossum@wur.nl", 
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-8673-2514")),
             person(given = "Willem", 
                    family = "Kruijer",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-7179-1733")),
             person(given = "Fred", 
                    family = "van Eeuwijk",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3672-2921")),
             person(given = "Martin", 
                    family = "Boer",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-1879-4588")),
             person(given = "Marcos",
                    family = "Malosetti",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-8150-1397")),
             person(given = "Daniela", 
                    family = "Bustos-Korts",
                    role = "ctb",
                    comment = c(ORCID = "0000-0003-3827-6726")),
             person(given = "Emilie", 
                    family = "Millet",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-2913-4892")),
             person(given = "Joao", 
                    family = "Paulo",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-4180-0763")),
             person(given = "Maikel", 
                    family = "Verouden",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-4893-3323")),
             person(given = "Ron", 
                    family = "Wehrens",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-8798-5599")),
             person(given = "Choazhi", 
                    family = "Zheng",
                    role = "ctb",
                    comment = c(ORCID = "0000-0001-6030-3933"))
            )
Description: Fast single trait Genome Wide Association Studies (GWAS) following 
    the method described in Kang et al. (2010), <doi:10.1038/ng.548>.        
    One of a series of statistical genetic packages for streamlining the 
    analysis of typical plant breeding experiments developed by Biometris.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 3.6)
Imports: data.table, ggplot2 (>= 3.0.0), LMMsolver, methods, rlang,
        Rcpp, sommer (>= 4.4.1)
Suggests: knitr, rmarkdown, officer, tinytest, snpStats, vcfR
VignetteBuilder: knitr
LinkingTo: Rcpp, RcppArmadillo
URL: https://biometris.github.io/statgenGWAS/index.html,
        https://github.com/Biometris/statgenGWAS/
BugReports: https://github.com/Biometris/statgenGWAS/issues
NeedsCompilation: yes
Packaged: 2026-02-19 15:06:39 UTC; rossu027
Author: Bart-Jan van Rossum [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8673-2514>),
  Willem Kruijer [aut] (ORCID: <https://orcid.org/0000-0001-7179-1733>),
  Fred van Eeuwijk [ctb] (ORCID: <https://orcid.org/0000-0003-3672-2921>),
  Martin Boer [ctb] (ORCID: <https://orcid.org/0000-0002-1879-4588>),
  Marcos Malosetti [ctb] (ORCID: <https://orcid.org/0000-0002-8150-1397>),
  Daniela Bustos-Korts [ctb] (ORCID:
    <https://orcid.org/0000-0003-3827-6726>),
  Emilie Millet [ctb] (ORCID: <https://orcid.org/0000-0002-2913-4892>),
  Joao Paulo [ctb] (ORCID: <https://orcid.org/0000-0002-4180-0763>),
  Maikel Verouden [ctb] (ORCID: <https://orcid.org/0000-0002-4893-3323>),
  Ron Wehrens [ctb] (ORCID: <https://orcid.org/0000-0002-8798-5599>),
  Choazhi Zheng [ctb] (ORCID: <https://orcid.org/0000-0001-6030-3933>)
Maintainer: Bart-Jan van Rossum <bart-jan.vanrossum@wur.nl>
Repository: CRAN
Date/Publication: 2026-02-19 15:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 10:38:16 UTC; unix
Archs: statgenGWAS.so.dSYM
