# Generated by roxygen2: do not edit by hand

S3method(plot,GWAS)
S3method(plot,gData)
S3method(print,summary.gData)
S3method(summary,GWAS)
S3method(summary,gData)
export(codeMarkers)
export(createGData)
export(kinship)
export(readPLINK)
export(readVcf)
export(runSingleTraitGwas)
import(data.table)
import(stats)
importFrom(Rcpp,sourceCpp)
importFrom(data.table,":=")
importFrom(graphics,plot)
importFrom(methods,as)
importFrom(rlang,.data)
importFrom(utils,capture.output)
importFrom(utils,hasName)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(statgenGWAS, .registration = TRUE)
